# -*- coding: utf-8 -*-
from Products.PlonePAS.testing import PRODUCTS_PLONEPAS_FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import re
import six
import unittest


class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if six.PY2:
            got = re.sub("IOError", "OSError", got)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(
        layered(
            doctest.DocFileSuite(
                'cookie_auth.rst',
                package='Products.PlonePAS.tests',
                optionflags=doctest.ELLIPSIS,
                checker=Py23DocChecker(),
            ),
            layer=PRODUCTS_PLONEPAS_FUNCTIONAL_TESTING
        )
    )
    suite.addTest(
        doctest.DocTestSuite(
            'Products.PlonePAS.utils',
            optionflags=doctest.ELLIPSIS,
            checker=Py23DocChecker(),
        )
    )
    return suite



# -*- coding: utf-8 -*-
from AccessControl.Permission import addPermission

AddGroups = 'Add Groups'
addPermission(AddGroups, default_roles=('Manager',))

ManageGroups = 'Manage Groups'
addPermission(ManageGroups, default_roles=('Manager',))

ViewGroups = 'View Groups'
addPermission(ViewGroups, default_roles=('Manager', 'Owner', 'Member'))

DeleteGroups = 'Delete Groups'
addPermission(DeleteGroups, default_roles=('Manager', ))

SetGroupOwnership = 'Set Group Ownership'
addPermission(SetGroupOwnership, default_roles=('Manager', 'Owner'))



# -*- coding: utf-8 -*-
"""
Add Mutable Property Sheets and Schema Mutable Property Sheets to PAS

also a property schema type registry which is extensible.

"""
from Products.CMFCore.interfaces import ISiteRoot
from Products.PlonePAS.interfaces.propertysheets import IMutablePropertySheet
from Products.PluggableAuthService.UserPropertySheet import UserPropertySheet
from zope.component import getUtility
from zope.interface import implementer

import six


class PropertyValueError(ValueError):
    pass


class PropertySchemaTypeMap(object):

    def __init__(self):
        self.tmap = {}
        self.tmap_order = []

    def addType(self, type_name, identifier, order=None):
        self.tmap[type_name] = identifier
        if order is not None and isinstance(order, int):
            self.tmap_order.insert(order, type_name)
        else:
            self.tmap_order.append(type_name)

    def getTypeFor(self, value):
        ptypes = [(ptype, self.tmap[ptype]) for ptype in self.tmap_order]
        for ptype, inspector in ptypes:
            if inspector(value):
                return ptype
        raise TypeError('Invalid property type: %s' % type(value))

    def validate(self, property_type, value):
        inspector = self.tmap[property_type]
        return inspector(value)

PropertySchema = PropertySchemaTypeMap()
PropertySchema.addType(
    'string',
    lambda x: x is None or isinstance(x, six.string_types)
)
PropertySchema.addType(
    'text',
    lambda x: x is None or isinstance(x, six.string_types)
)
PropertySchema.addType(
    'boolean',
    lambda x: 1  # anything can be boolean
)
PropertySchema.addType(
    'int',
    lambda x: x is None or isinstance(x, int)
)
PropertySchema.addType(
    'long',
    lambda x: x is None or isinstance(x, long)
)
PropertySchema.addType(
    'float',
    lambda x: x is None or isinstance(x, float)
)
PropertySchema.addType(
    'lines',
    lambda x: x is None or isinstance(x, (tuple, list))
)
PropertySchema.addType(
    'selection',
    lambda x: x is None or isinstance(x, six.string_types)
)
PropertySchema.addType(
    'multiple selection',
    lambda x: x is None or isinstance(x, (tuple, list))
)
PropertySchema.addType(
    'date',
    lambda x: 1
)
validateValue = PropertySchema.validate


@implementer(IMutablePropertySheet)
class MutablePropertySheet(UserPropertySheet):

    def validateProperty(self, id, value):
        if id not in self._properties:
            raise PropertyValueError('No such property found on this schema')

        proptype = self.getPropertyType(id)
        if not validateValue(proptype, value):
            raise PropertyValueError(
                "Invalid value (%s) for property '%s' of type %s" %
                (value, id, proptype)
            )

    def setProperty(self, user, id, value):
        self.validateProperty(id, value)

        self._properties[id] = value
        self._properties = self._properties

        # cascade to plugin
        provider = self._getPropertyProviderForUser(user)
        provider.setPropertiesForUser(user, self)

    def setProperties(self, user, mapping):
        prop_keys = self._properties.keys()
        prop_update = mapping.copy()

        for key, value in tuple(prop_update.items()):
            if key not in prop_keys:
                prop_update.pop(key)
                continue
            self.validateProperty(key, value)

        self._properties.update(prop_update)

        # cascade to plugin
        provider = self._getPropertyProviderForUser(user)
        provider.setPropertiesForUser(user, self)

    def _getPropertyProviderForUser(self, user):
        # XXX This assumes that the acl_users that we want is in the portal
        # root. This may not always be the case.
        portal = getUtility(ISiteRoot)
        return portal.acl_users._getOb(self._id)


class SchemaMutablePropertySheet(MutablePropertySheet):
    pass



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import Products.PlonePAS


class ProductsPlonepasLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=Products.PlonePAS)
        z2.installProduct(app, 'Products.PlonePAS')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'Products.PlonePAS:PlonePAS')
        # setRoles(portal, TEST_USER_ID, ['Manager'])
        from Products.CMFPlone.utils import _createObjectByType
        _createObjectByType('Folder', portal, id='Members')
        mtool = portal.portal_membership
        if not mtool.getMemberareaCreationFlag():
            mtool.setMemberareaCreationFlag()
        mtool.createMemberArea(TEST_USER_ID)
        if mtool.getMemberareaCreationFlag():
            mtool.setMemberareaCreationFlag()

        _createObjectByType('Folder', portal, id='folder')


PRODUCTS_PLONEPAS_FIXTURE = ProductsPlonepasLayer()


PRODUCTS_PLONEPAS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PRODUCTS_PLONEPAS_FIXTURE,),
    name='ProductsPlonepasLayer:IntegrationTesting',
)


PRODUCTS_PLONEPAS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PRODUCTS_PLONEPAS_FIXTURE,),
    name='ProductsPlonepasLayer:FunctionalTesting',
)



# -*- coding: utf-8 -*-
""" Class: ExtendedCookieAuthHelper

Simply extends the standard CookieAuthHelper provided via regular
PluggableAuthService but overrides the updateCookie mechanism to
provide similar functionality as CookieCrumbler does... by giving
the portal the ability to provide a setAuthCookie method.
"""
from AccessControl.SecurityInfo import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_parent
from AccessControl.class_init import InitializeClass
from App.special_dtml import DTMLFile
from DateTime import DateTime
from plone.registry.interfaces import IRegistry
from Products.PluggableAuthService.interfaces.authservice import \
    IPluggableAuthService
from Products.PluggableAuthService.interfaces.plugins \
    import IChallengePlugin
from Products.PluggableAuthService.interfaces.plugins \
    import ICredentialsResetPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import ICredentialsUpdatePlugin
from Products.PluggableAuthService.interfaces.plugins \
    import ILoginPasswordHostExtractionPlugin
from Products.PluggableAuthService.plugins.CookieAuthHelper \
    import CookieAuthHelper as BasePlugin
from six.moves.urllib.parse import quote
from zope.interface import implementer
from zope.component import getUtility


def manage_addExtendedCookieAuthHelper(self, id, title='',
                                       RESPONSE=None, **kw):
    """Create an instance of a extended cookie auth helper.
    """

    self = self.this()

    o = ExtendedCookieAuthHelper(id, title, **kw)
    self._setObject(o.getId(), o)
    o = getattr(aq_base(self), id)

    if RESPONSE is not None:
        RESPONSE.redirect('manage_workspace')

manage_addExtendedCookieAuthHelperForm = DTMLFile(
    "../zmi/ExtendedCookieAuthHelperForm",
    globals()
)


@implementer(
    ILoginPasswordHostExtractionPlugin,
    IChallengePlugin,
    ICredentialsUpdatePlugin,
    ICredentialsResetPlugin
)
class ExtendedCookieAuthHelper(BasePlugin):
    """Multi-plugin which adds ability to override the updating of cookie via
    a setAuthCookie method/script.
    """

    meta_type = 'Extended Cookie Auth Helper'
    security = ClassSecurityInfo()

    @security.private
    def updateCredentials(self, request, response, login, new_password):
        """Override standard updateCredentials method
        """
        cookie_val = self.get_cookie_value(login, new_password)
        kw = {}
        registry = getUtility(IRegistry)
        length = registry.get('plone.auth_cookie_length', '0')
        try:
            length = int(length)
        except ValueError:
            length = 0
        if length:
            kw.update(expires=(DateTime() + length).toZone('GMT').rfc822())
        response.setCookie(
            self.cookie_name, quote(cookie_val), path='/', **kw)

    @security.public
    def login(self):
        """Set a cookie and redirect to the url that we tried to
        authenticate against originally.

        Override standard login method to avoid calling
        'return response.redirect(came_from)' as there is additional
        processing to ignore known bad come_from templates at
        login_next.cpy script.
        """
        request = self.REQUEST
        response = request['RESPONSE']

        password = request.get('__ac_password', '')

        user = getSecurityManager().getUser()
        login = user.getUserName()
        user_pas = aq_parent(user)

        if IPluggableAuthService.providedBy(user_pas):
            # Delegate to the users own PAS if possible
            user_pas.updateCredentials(request, response, login, password)
        else:
            # User does not originate from a PAS user folder, so lets try
            # to do our own thing.
            # XXX Perhaps we should do nothing here; test with pure User
            # Folder!
            pas_instance = self._getPAS()
            if pas_instance is not None:
                pas_instance.updateCredentials(request, response, login,
                                               password)


InitializeClass(ExtendedCookieAuthHelper)
